extern crate serde_json;
use crate::parser;
use rayon::prelude::*;
use serde::ser::{Serialize, SerializeStruct, Serializer};
//use serde_json::json;

impl Serialize for parser::QryData {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let mut state = serializer.serialize_struct("parser::QryData", 11)?;
        state.serialize_field("time", &self.time)?;
        state.serialize_field("ether_header.ether_dhost", &self.ether_header.ether_dhost)?;
        state.serialize_field("ether_header.ether_shost", &self.ether_header.ether_shost)?;
        state.serialize_field("ether_header.ether_type", &self.ether_header.ether_type)?;
        state.serialize_field("ipv4_header", &self.ipv4_header)?;
        state.serialize_field("ipv6_header", &self.ipv6_header)?;
        state.serialize_field("tcp_header", &self.tcp_header)?;
        state.serialize_field("udp_header", &self.udp_header)?;
        state.serialize_field("arp_header", &self.arp_header)?;
        state.serialize_field("data", &self.data)?;
        state.serialize_field("reg_res", &self.reg_res)?;
        state.end()
    }
}



pub fn serialize_packets(v: Vec<parser::QryData>) -> Vec<serde_json::Value> {

    /* rayon parallelized */
    // TODO: Benchmark
    let packets_serialized = v.par_iter().map(|x| serde_json::to_value(x).unwrap()).collect();
     //let packets_serialized: Vec<serde_json::Value> = v.par_iter().map(|x| json!(x)).collect();

    packets_serialized
}


// This is way faster than serialize_packets() but can not figure out how to properly select parts of the json from db.
#[allow(dead_code)]
pub fn serialize_packets_as_string(v: Vec<parser::QryData>) -> Vec<serde_json::Value> {
    let mut packets_serialized: Vec<serde_json::Value> = Vec::with_capacity(v.len() * 2);
     for packet in v.iter() {
         packets_serialized.push(serde_json::Value::String(serde_json::to_string(&packet).unwrap()));
     }

    packets_serialized
}

#[allow(dead_code)]
fn smallest_prime_divisor(remainder: usize ) -> usize {
    let smallest_divisor: usize = (2..(remainder/2)).into_par_iter().find_first(|x| remainder % x == 0).unwrap();
    smallest_divisor
}
